use clap::{Parser, Subcommand};

mod mnemonic;
mod provision;
mod shard;
mod recover;

/// The Kitchen Sink of Entropy.
#[derive(Parser, Clone, Debug)]
#[command(author, version, about, long_about = None)]
pub struct Keyfork {
    // Global options
    #[command(subcommand)]
    pub command: KeyforkCommands,
}

#[derive(Subcommand, Clone, Debug)]
pub enum KeyforkCommands {
    /// Mnemonic generation and persistence utilities.
    Mnemonic(mnemonic::Mnemonic),

    /// Splitting and combining secrets using Shamir's Secret Sharing.
    Shard(shard::Shard),

    /// Derive and deploy keys.
    #[command(subcommand_negates_reqs(true))]
    Provision(provision::Provision),

    /// Recover a seed using a recovery mechanism and begin the Keyfork daemon.
    Recover(recover::Recover),
}

impl KeyforkCommands {
    pub fn handle(&self, keyfork: &Keyfork) -> Result<(), Box<dyn std::error::Error>> {
        match self {
            KeyforkCommands::Mnemonic(m) => {
                let response = m.command.handle(m, keyfork)?;
                println!("{response}");
            }
            KeyforkCommands::Shard(s) => {
                // TODO: When actually fleshing out, this takes a `Read` and a `Write`
                s.command.handle(s, keyfork)?;
            }
            KeyforkCommands::Provision(p) => {
                p.handle(keyfork)?;
            }
            KeyforkCommands::Recover(r) => {
                r.handle(keyfork)?;
            }
        }
        Ok(())
    }
}
